; =============== S U B R O U T I N E =======================================


InitSoundEngine:
	ldx	#$3F
	lda	#$FF

loc_F3B2:
	sta	instPtrLo,x
	dex
	bpl	loc_F3B2
	inx
	stx	apuStatusCopy
	stx	$4015	; mute all sound channels
	rts
; End of function InitSoundEngine


; =============== S U B R O U T I N E =======================================

; In: A - Sound to play

PlaySound:
	asl	a
	tax
	lda	soundPtrTable,x
	sta	!$E
	lda	soundPtrTable+1,x
	sta	!$F
	ldy	#0
	lda	($E),y
	sta	!$D

loc_F3D5:
	iny
	lda	($E),y
	tax
	lda	#1
	sta	instTimer,x
	lda	#$FF
	sta	instCtrlRegsSet,x

loc_F3E3:
	iny
	lda	($E),y
	sta	instPtrLo,x
	txa
	clc
	adc	#6
	tax
	cpx	#$24
	bcc	loc_F3E3
	dec	!$D
	bne	loc_F3D5

locret_F3F7:
	rts
; End of function PlaySound

; ---------------------------------------------------------------------------
; START OF FUNCTION CHUNK FOR WaitVblank

RunSoundEngine:
	lda	#0
	ldx	#3

loc_F3FC:
	sta	channelsInUse,x
	dex
	bpl	loc_F3FC
	ldx	#5

loc_F404:
	jsr	RunSoundInstrument
	dex
	bne	loc_F404	; fall through after loop ends (so there's 6 instruments total)
; END OF FUNCTION CHUNK FOR WaitVblank

; =============== S U B R O U T I N E =======================================


RunSoundInstrument:
	lda	instCursor,x
	cmp	#$FF	; check for "instrument not playing" flag
	beq	locret_F3F7
	dec	instTimer,x
	beq	soundSetupReadPtr
	lda	#$6F	; write "instrument is still playing a note" flag
	sta	!$A
	jmp	loc_F51F
; ---------------------------------------------------------------------------

soundSetupReadPtr:
	lda	#0
	sta	!$F
	lda	instCursor,x
	asl	a
	rol	!$F
	clc
	adc	instPtrLo,x
	sta	!$E
	lda	!$F
	adc	instPtrHi,x
	sta	!$F
	ldy	#0

soundReadCmd:
	lda	($E),y
	cmp	#$FF
	bne	loc_F457
	sta	instCursor,x
	sta	instLoop,x
	lda	instChannel,x
	tay
	lda	#0
	sta	channelsInUse,y
	jsr	DisableChannel
	jmp	loc_F55E
; ---------------------------------------------------------------------------

loc_F457:
	sta	!$C
	iny
	lda	($E),y
	sta	!$D
	lda	!$C
	cmp	#$C0
	bcs	loc_F4C5	; >= C0
	cmp	#$B0
	bcs	loc_F48E	; >= B0
	cmp	#$A0
	bcc	loc_F4C5	; < A0
	and	#1
	bne	loc_F47C
	lda	!$D	; A0: Set register 0
	sta	instReg0,x
	jmp	loc_F482
; ---------------------------------------------------------------------------

loc_F47C:
	lda	!$D	; A1: Set register 1
	sta	instReg1,x

loc_F482:
	iny
	inc	instCursor,x
	lda	#$FF
	sta	instCtrlRegsSet,x
	jmp	soundReadCmd
; ---------------------------------------------------------------------------

loc_F48E:
	cmp	#$BF
	bne	loc_F49B
	lda	!$D	; BF: Set instrument cursor
	sta	instCursor,x
	jmp	soundSetupReadPtr
; ---------------------------------------------------------------------------

loc_F49B:
	lda	instLoop,x
	beq	loc_F4BB
	cmp	#$FF
	beq	loc_F4AA
	dec	instLoop,x	; loop in progress: decrement the counter & set the cursor
	jmp	loc_F4B2
; ---------------------------------------------------------------------------

loc_F4AA:
; B0-BE: Set up instrument loop (the way this is structured,
; you should use it AFTER the notes you want to loop)
	lda	!$C
	and	#$F
	sta	instLoop,x

loc_F4B2:
	lda	!$D
	sta	instCursor,x
	jmp	soundSetupReadPtr
; ---------------------------------------------------------------------------

loc_F4BB:
	dec	instLoop,x	; loop over: set to $FF (no loop) and move on
	inc	instCursor,x
	iny
	jmp	soundReadCmd
; ---------------------------------------------------------------------------

loc_F4C5:
	inc	instCursor,x
	lda	instChannel,x
	ldy	!$C
	cmp	#3
	beq	loc_F509	; noise channel? set registers directly
	lda	!$C
	and	#$F
	cmp	#$C	; >= C? key off
	bcs	loc_F500
	asl	a
	tay
	lda	noteTbl+1,y
	sta	!$A
	lda	noteTbl,y
	sta	!$B
	lda	!$C
	and	#$F0
	cmp	#$A0
	lsr	a
	lsr	a
	lsr	a
	lsr	a
	tay

loc_F4F5:
	lsr	!$B
	ror	!$A
	dey
	bpl	loc_F4F5
	bmi	loc_F511

loc_F500:
	lda	instChannel,x
	tay
	jsr	DisableChannel
	ldy	#$6F

loc_F509:
	sty	!$A
	ldy	#0
	sty	!$B

loc_F511:
	lda	!$D
	sta	instTimer,x
	lda	!$B
	ora	#8
	sta	!$B

loc_F51F:
	lda	instChannel,x
	tay
	lda	channelsInUse,y
	bne	loc_F55E
	lda	#$FF
	sta	channelsInUse,y
	lda	!$A
	cmp	#$6F
	beq	locret_F55D
	jsr	EnableChannel
	tya
	asl	a
	asl	a
	tay
	lda	instCtrlRegsSet,x
	beq	loc_F54C
	lda	instReg1,x
	sta	$4001,y
	lda	instReg0,x
	sta	$4000,y

loc_F54C:
	lda	!$A
	sta	$4002,y
	lda	!$B
	sta	$4003,y
	lda	#0
	sta	instCtrlRegsSet,x

locret_F55D:
	rts
; ---------------------------------------------------------------------------

loc_F55E:
; if the instrument lost access to its channel, it needs
; to reset its control registers when it regains access
	lda	#$FF
	sta	instCtrlRegsSet,x
	rts
; End of function RunSoundInstrument


; =============== S U B R O U T I N E =======================================

; In: Y: Channel number to disable

DisableChannel:
	lda	apuStatusCopy
	and	channelOffVals,y
	sta	apuStatusCopy
	sta	$4015
	rts
; End of function DisableChannel


; =============== S U B R O U T I N E =======================================

; In: Y: Channel number to enable

EnableChannel:
	lda	apuStatusCopy
	ora	channelOnVals,y
	sta	apuStatusCopy
	sta	$4015
	rts
; End of function EnableChannel

; ---------------------------------------------------------------------------
noteTbl:
	db	$0D
	db	$5C
	db	$0C
	db	$9C
	db	$0B
	db	$E8
	db	$0B
	db	$3C
	db	$0A
	db	$9A
	db	$0A
	db	$02
	db	$09
	db	$72
	db	$08
	db	$EA
	db	$08
	db	$6A
	db	$07
	db	$F2
	db	$07
	db	$80
	db	$07
	db	$14
channelOnVals:
	db	$01	; bit 0 = pulse 1
	db	$02	; bit 1 = pulse 2
	db	$04	; bit 2 = triangle
	db	$08	; bit 3 = noise
channelOffVals:
	db	$0E	; bit 0 clear
	db	$0D	; bit 1 clear
	db	$0B	; bit 2 clear
	db	$07	; bit 3 clear
soundPtrTable:
	dw	$400	; title screen music
	dw	$420	; ending music
	dw	stageStartMusic
	dw	levelClearMusic
	dw	bossActiveMusic
	dw	itemRoomMusic
	dw	gameOverMusic
	dw	caveMusic
	dw	overworldMusic
	dw	overworldMusic
	dw	peraskullSound
	dw	fireballSound
	dw	castleMusic
	dw	swordSound
	dw	menuSound
	dw	luciaHitSound
	dw	magicBombSound
	dw	luciaJumpSound
	dw	enemyHitSound
	dw	magicBombSplitSound
	dw	shieldBallSound
	dw	nomajiJumpSound
	dw	boundBallSound
	dw	yokkoChanSound
	dw	enemyKilledSound
	dw	itemPickupSound
	dw	bossKilledSound
	dw	pauseSound
	dw	weaponSelectSound
	dw	flameSwordSound
	dw	nyuruSound
	dw	joyraimaSound
stageStartMusic:
	db	$02
	db	$01
	dw	unk_F72F
	db	$00
	db	$00
	db	$00
	db	$4F
	db	$00
	dw	unk_F762
	db	$01
	db	$00
	db	$00
	db	$CF
levelClearMusic:
	db	$02
	db	$01
	dw	unk_F795
	db	$00
	db	$00
	db	$00
	db	$4F
	db	$00
	dw	unk_F7AC
	db	$01
	db	$00
	db	$00
	db	$CF
bossActiveMusic:
	db	$02
	db	$01
	dw	unk_FB78
	db	$03
	db	$00
	db	$00
	db	$00
	db	$00
	dw	unk_FB64
	db	$00
	db	$00
	db	$88
	db	$5F
itemRoomMusic:
	db	$01
	db	$00
	dw	unk_FB64
	db	$00
	db	$00
	db	$88
	db	$96
gameOverMusic:
	db	$02
	db	$01
	dw	unk_F7C3
	db	$00
	db	$00
	db	$00
	db	$5F
	db	$00
	dw	unk_F7CE
	db	$01
	db	$00
	db	$88
	db	$9F
caveMusic:
	db	$04
	db	$00
	dw	unk_F7EF
	db	$01
	db	$00
	db	$00
	db	$CF
	db	$01
	dw	unk_F7D9
	db	$00
	db	$00
	db	$00
	db	$4F
	db	$02
	dw	unk_F805
	db	$02
	db	$00
	db	$00
	db	$10
	db	$03
	dw	unk_F81B
	db	$03
	db	$00
	db	$00
	db	$00
overworldMusic:
	db	$03
	db	$02
	dw	unk_F81F
	db	$00
	db	$00
	db	$00
	db	$8F
	db	$01
	dw	unk_F925
	db	$01
	db	$00
	db	$00
	db	$CF
	db	$00
	dw	unk_FA61
	db	$02
	db	$00
	db	$00
	db	$20
peraskullSound:
	db	$01
	db	$04
	dw	unk_FF27
	db	$01
	db	$00
	db	$81
	db	$5F
fireballSound:
	db	$01
	db	$03
	dw	unk_FBC0
	db	$03
	db	$00
	db	$00
	db	$8F
castleMusic:
	db	$04
	db	$00
	dw	unk_FD9B
	db	$01
	db	$00
	db	$88
	db	$DF
	db	$01
	dw	unk_FC89
	db	$02
	db	$00
	db	$00
	db	$FF
	db	$02
	dw	unk_FBD5
	db	$00
	db	$00
	db	$00
	db	$4F
	db	$03
	dw	unk_FEBD
	db	$03
	db	$00
	db	$00
	db	$00
swordSound:
	db	$01
	db	$04
	dw	unk_FF46
	db	$01
	db	$00
	db	$8B
	db	$1F
menuSound:
	db	$01
	db	$04
	dw	unk_FF27
	db	$01
	db	$00
	db	$00
	db	$81
luciaHitSound:
	db	$01
	db	$05
	dw	unk_FF2A
	db	$01
	db	$00
	db	$83
	db	$9F
magicBombSound:
	db	$01
	db	$04
	dw	unk_FF33
	db	$01
	db	$00
	db	$82
	db	$DF
luciaJumpSound:
	db	$02
	db	$05
	dw	unk_FBC9
	db	$03
	db	$00
	db	$00
	db	$00
	db	$04
	dw	unk_FF46
	db	$01
	db	$00
	db	$8B
	db	$9F
enemyHitSound:
	db	$01
	db	$05
	dw	unk_FF4D
	db	$03
	db	$00
	db	$00
	db	$8F
magicBombSplitSound:
	db	$01
	db	$04
	dw	unk_FF77
	db	$01
	db	$00
	db	$9F
	db	$8F
shieldBallSound:
	db	$01
	db	$04
	dw	unk_FF5A
	db	$01
	db	$00
	db	$93
	db	$95
nomajiJumpSound:
	db	$01
	db	$04
	dw	unk_FF46
	db	$01
	db	$00
	db	$89
	db	$DF
boundBallSound:
	db	$01
	db	$04
	dw	unk_FF86
	db	$01
	db	$00
	db	$9B
	db	$9F
yokkoChanSound:
	db	$01
	db	$03
	dw	unk_FB61
	db	$01
	db	$00
	db	$84
	db	$9F
enemyKilledSound:
	db	$01
	db	$04
	dw	unk_FF46
	db	$01
	db	$00
	db	$82
	db	$DF
itemPickupSound:
	db	$01
	db	$04
	dw	unk_FB80
	db	$01
	db	$00
	db	$8C
	db	$9F
bossKilledSound:
	db	$02
	db	$05
	dw	unk_FB87
	db	$01
	db	$00
	db	$83
	db	$DF
	db	$04
	dw	unk_FB87
	db	$00
	db	$00
	db	$83
	db	$9F
pauseSound:
	db	$01
	db	$03
	dw	unk_FBA6
	db	$01
	db	$00
	db	$00
	db	$DF
weaponSelectSound:
	db	$02
	db	$03
	dw	unk_FBB3
	db	$01
	db	$00
	db	$00
	db	$DF
	db	$02
	dw	unk_FBB6
	db	$02
	db	$00
	db	$00
	db	$40
flameSwordSound:
	db	$01
	db	$04
	dw	unk_FBB9
	db	$03
	db	$00
	db	$00
	db	$8F
nyuruSound:
	db	$01
	db	$04
	dw	unk_FB80
	db	$01
	db	$00
	db	$8A
	db	$DF
joyraimaSound:
	db	$01
	db	$05
	dw	unk_FBCC
	db	$01
	db	$00
	db	$86
	db	$DF
unk_F72F:
	db	$29
	db	$0C
	db	$30
	db	$0C
	db	$35
	db	$06
	db	$37
	db	$06
	db	$39
	db	$06
	db	$35
	db	$12
	db	$30
	db	$0C
	db	$29
	db	$0C
	db	$30
	db	$0C
	db	$27
	db	$0C
	db	$2A
	db	$0C
	db	$33
	db	$06
	db	$35
	db	$06
	db	$37
	db	$06
	db	$33
	db	$12
	db	$2A
	db	$0C
	db	$27
	db	$0C
	db	$2A
	db	$0C
	db	$26
	db	$24
	db	$26
	db	$06
	db	$26
	db	$06
	db	$27
	db	$24
	db	$27
	db	$06
	db	$27
	db	$06
	db	$26
	db	$60
	db	$FF
unk_F762:
	db	$25
	db	$0C
	db	$29
	db	$0C
	db	$29
	db	$06
	db	$2A
	db	$06
	db	$30
	db	$06
	db	$29
	db	$12
	db	$29
	db	$0C
	db	$25
	db	$0C
	db	$29
	db	$0C
	db	$23
	db	$0C
	db	$27
	db	$0C
	db	$27
	db	$06
	db	$28
	db	$06
	db	$2A
	db	$06
	db	$27
	db	$12
	db	$27
	db	$0C
	db	$23
	db	$0C
	db	$27
	db	$0C
	db	$19
	db	$24
	db	$19
	db	$06
	db	$19
	db	$06
	db	$1A
	db	$24
	db	$1A
	db	$06
	db	$1A
	db	$06
	db	$19
	db	$60
	db	$FF
unk_F795:
	db	$2B
	db	$0C
	db	$32
	db	$0C
	db	$37
	db	$0C
	db	$39
	db	$0C
	db	$3B
	db	$0C
	db	$37
	db	$0C
	db	$32
	db	$0C
	db	$6F
	db	$0C
	db	$32
	db	$0C
	db	$B2
	db	$08
	db	$32
	db	$48
	db	$FF
unk_F7AC:
	db	$27
	db	$0C
	db	$2B
	db	$0C
	db	$2B
	db	$0C
	db	$30
	db	$0C
	db	$32
	db	$0C
	db	$2B
	db	$0C
	db	$2B
	db	$0C
	db	$6F
	db	$0C
	db	$2B
	db	$0C
	db	$B2
	db	$08
	db	$2B
	db	$48
	db	$FF
unk_F7C3:
	db	$1B
	db	$20
	db	$22
	db	$20
	db	$21
	db	$20
	db	$20
	db	$20
	db	$1B
	db	$80
	db	$FF
unk_F7CE:
	db	$07
	db	$20
	db	$0B
	db	$20
	db	$0A
	db	$20
	db	$09
	db	$20
	db	$07
	db	$80
	db	$FF
unk_F7D9:
	db	$24
	db	$30
	db	$23
	db	$30
	db	$22
	db	$30
	db	$21
	db	$30
	db	$B0
	db	$00
	db	$29
	db	$30
	db	$28
	db	$30
	db	$27
	db	$30
	db	$26
	db	$30
	db	$B0
	db	$05
	db	$BF
	db	$00
unk_F7EF:
	db	$20
	db	$30
	db	$1B
	db	$30
	db	$1A
	db	$30
	db	$19
	db	$30
	db	$B0
	db	$00
	db	$25
	db	$30
	db	$24
	db	$30
	db	$23
	db	$30
	db	$22
	db	$30
	db	$B0
	db	$05
	db	$BF
	db	$00
unk_F805:
	db	$29
	db	$0C
	db	$2A
	db	$0C
	db	$2B
	db	$0C
	db	$30
	db	$0C
	db	$B6
	db	$00
	db	$32
	db	$0C
	db	$33
	db	$0C
	db	$34
	db	$0C
	db	$35
	db	$0C
	db	$B6
	db	$05
	db	$BF
	db	$00
unk_F81B:
	db	$04
	db	$0C
	db	$BF
	db	$00
unk_F81F:
	db	$26
	db	$1E
	db	$27
	db	$32
	db	$29
	db	$0A
	db	$6F
	db	$0A
	db	$32
	db	$0A
	db	$29
	db	$1E
	db	$26
	db	$14
	db	$2A
	db	$28
	db	$29
	db	$14
	db	$27
	db	$0A
	db	$29
	db	$1E
	db	$27
	db	$14
	db	$26
	db	$0A
	db	$27
	db	$1E
	db	$26
	db	$1E
	db	$27
	db	$32
	db	$29
	db	$0A
	db	$6F
	db	$0A
	db	$32
	db	$0A
	db	$29
	db	$1E
	db	$32
	db	$14
	db	$32
	db	$14
	db	$30
	db	$14
	db	$2B
	db	$14
	db	$30
	db	$0A
	db	$29
	db	$3C
	db	$29
	db	$05
	db	$29
	db	$05
	db	$31
	db	$05
	db	$31
	db	$05
	db	$34
	db	$05
	db	$34
	db	$05
	db	$36
	db	$32
	db	$37
	db	$0A
	db	$36
	db	$0A
	db	$32
	db	$0A
	db	$34
	db	$0A
	db	$6F
	db	$0A
	db	$32
	db	$0A
	db	$2A
	db	$1E
	db	$30
	db	$14
	db	$29
	db	$32
	db	$26
	db	$0A
	db	$27
	db	$0A
	db	$29
	db	$0A
	db	$2B
	db	$0A
	db	$27
	db	$0A
	db	$29
	db	$0A
	db	$2B
	db	$0A
	db	$31
	db	$0A
	db	$29
	db	$0A
	db	$31
	db	$0A
	db	$34
	db	$0A
	db	$36
	db	$32
	db	$37
	db	$0A
	db	$36
	db	$0A
	db	$32
	db	$0A
	db	$34
	db	$0A
	db	$6F
	db	$0A
	db	$32
	db	$0A
	db	$2A
	db	$1E
	db	$30
	db	$14
	db	$29
	db	$1E
	db	$32
	db	$1E
	db	$36
	db	$14
	db	$39
	db	$3C
	db	$6F
	db	$0A
	db	$2B
	db	$0A
	db	$2B
	db	$14
	db	$34
	db	$0A
	db	$32
	db	$0A
	db	$6F
	db	$1E
	db	$31
	db	$14
	db	$31
	db	$14
	db	$31
	db	$0A
	db	$32
	db	$14
	db	$34
	db	$14
	db	$34
	db	$55
	db	$31
	db	$05
	db	$29
	db	$05
	db	$27
	db	$05
	db	$31
	db	$05
	db	$29
	db	$05
	db	$27
	db	$05
	db	$24
	db	$05
	db	$29
	db	$05
	db	$27
	db	$05
	db	$24
	db	$05
	db	$21
	db	$05
	db	$27
	db	$05
	db	$24
	db	$05
	db	$21
	db	$05
	db	$19
	db	$05
	db	$22
	db	$14
	db	$32
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$6F
	db	$0A
	db	$22
	db	$0A
	db	$27
	db	$14
	db	$2A
	db	$0A
	db	$27
	db	$0A
	db	$32
	db	$0A
	db	$27
	db	$0A
	db	$37
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$14
	db	$32
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$6F
	db	$0A
	db	$22
	db	$0A
	db	$25
	db	$0A
	db	$25
	db	$0A
	db	$30
	db	$0A
	db	$25
	db	$0A
	db	$2B
	db	$0A
	db	$2B
	db	$0A
	db	$27
	db	$0A
	db	$2B
	db	$0A
	db	$29
	db	$32
	db	$29
	db	$14
	db	$29
	db	$0A
	db	$29
	db	$0A
	db	$6F
	db	$14
	db	$21
	db	$0A
	db	$22
	db	$14
	db	$24
	db	$14
	db	$BF
	db	$00
unk_F925:
	db	$22
	db	$1E
	db	$23
	db	$32
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$26
	db	$0A
	db	$26
	db	$1E
	db	$22
	db	$14
	db	$20
	db	$28
	db	$20
	db	$14
	db	$20
	db	$0A
	db	$20
	db	$1E
	db	$1B
	db	$14
	db	$22
	db	$0A
	db	$23
	db	$1E
	db	$22
	db	$1E
	db	$23
	db	$32
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$26
	db	$0A
	db	$26
	db	$1E
	db	$26
	db	$14
	db	$25
	db	$14
	db	$25
	db	$14
	db	$25
	db	$14
	db	$25
	db	$0A
	db	$25
	db	$32
	db	$24
	db	$0A
	db	$27
	db	$05
	db	$27
	db	$05
	db	$24
	db	$05
	db	$24
	db	$05
	db	$21
	db	$05
	db	$21
	db	$05
	db	$29
	db	$0A
	db	$29
	db	$0A
	db	$26
	db	$0A
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$2B
	db	$0A
	db	$29
	db	$0A
	db	$27
	db	$0A
	db	$2A
	db	$0A
	db	$6F
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$24
	db	$0A
	db	$24
	db	$0A
	db	$27
	db	$0A
	db	$27
	db	$0A
	db	$6F
	db	$14
	db	$26
	db	$05
	db	$26
	db	$05
	db	$26
	db	$05
	db	$26
	db	$05
	db	$26
	db	$0A
	db	$22
	db	$0A
	db	$24
	db	$0A
	db	$26
	db	$0A
	db	$27
	db	$0A
	db	$24
	db	$0A
	db	$26
	db	$0A
	db	$27
	db	$0A
	db	$29
	db	$0A
	db	$24
	db	$0A
	db	$29
	db	$0A
	db	$31
	db	$0A
	db	$29
	db	$0A
	db	$29
	db	$0A
	db	$26
	db	$0A
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$2B
	db	$0A
	db	$29
	db	$0A
	db	$26
	db	$0A
	db	$2A
	db	$0A
	db	$6F
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$24
	db	$0A
	db	$24
	db	$0A
	db	$27
	db	$0A
	db	$27
	db	$0A
	db	$6F
	db	$14
	db	$26
	db	$05
	db	$26
	db	$05
	db	$26
	db	$05
	db	$26
	db	$05
	db	$6F
	db	$14
	db	$29
	db	$05
	db	$29
	db	$05
	db	$29
	db	$05
	db	$29
	db	$05
	db	$6F
	db	$14
	db	$22
	db	$05
	db	$22
	db	$05
	db	$22
	db	$05
	db	$22
	db	$05
	db	$22
	db	$0A
	db	$6F
	db	$0A
	db	$22
	db	$05
	db	$24
	db	$05
	db	$25
	db	$05
	db	$26
	db	$05
	db	$27
	db	$14
	db	$27
	db	$0A
	db	$27
	db	$0A
	db	$6F
	db	$1E
	db	$27
	db	$14
	db	$27
	db	$14
	db	$27
	db	$0A
	db	$27
	db	$14
	db	$27
	db	$14
	db	$29
	db	$5A
	db	$21
	db	$0A
	db	$14
	db	$0A
	db	$21
	db	$0A
	db	$14
	db	$0A
	db	$21
	db	$0A
	db	$14
	db	$0A
	db	$21
	db	$0A
	db	$19
	db	$14
	db	$26
	db	$0A
	db	$19
	db	$0A
	db	$19
	db	$0A
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$19
	db	$0A
	db	$1A
	db	$14
	db	$22
	db	$0A
	db	$1A
	db	$0A
	db	$27
	db	$0A
	db	$22
	db	$0A
	db	$2A
	db	$0A
	db	$1A
	db	$0A
	db	$19
	db	$14
	db	$26
	db	$0A
	db	$19
	db	$0A
	db	$19
	db	$0A
	db	$26
	db	$0A
	db	$6F
	db	$0A
	db	$19
	db	$0A
	db	$20
	db	$0A
	db	$20
	db	$0A
	db	$20
	db	$0A
	db	$20
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$0A
	db	$22
	db	$32
	db	$22
	db	$14
	db	$22
	db	$0A
	db	$21
	db	$0A
	db	$6F
	db	$14
	db	$19
	db	$0A
	db	$1B
	db	$14
	db	$21
	db	$14
	db	$BF
	db	$00
unk_FA61:
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$B6
	db	$00
	db	$25
	db	$0A
	db	$35
	db	$0A
	db	$B4
	db	$03
	db	$29
	db	$0A
	db	$39
	db	$0A
	db	$39
	db	$0A
	db	$39
	db	$0A
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$B6
	db	$0A
	db	$25
	db	$0A
	db	$35
	db	$0A
	db	$B2
	db	$0D
	db	$27
	db	$0A
	db	$37
	db	$0A
	db	$B0
	db	$10
	db	$29
	db	$0A
	db	$39
	db	$0A
	db	$B1
	db	$14
	db	$32
	db	$0A
	db	$B0
	db	$16
	db	$42
	db	$0A
	db	$32
	db	$0A
	db	$6F
	db	$28
	db	$37
	db	$0A
	db	$B0
	db	$1B
	db	$3A
	db	$0A
	db	$37
	db	$0A
	db	$40
	db	$0A
	db	$37
	db	$0A
	db	$42
	db	$0A
	db	$37
	db	$0A
	db	$32
	db	$0A
	db	$B0
	db	$23
	db	$42
	db	$0A
	db	$32
	db	$0A
	db	$6F
	db	$28
	db	$27
	db	$0A
	db	$37
	db	$0A
	db	$B0
	db	$28
	db	$29
	db	$0A
	db	$39
	db	$0A
	db	$B0
	db	$2B
	db	$32
	db	$0A
	db	$B0
	db	$2E
	db	$42
	db	$0A
	db	$32
	db	$14
	db	$32
	db	$0A
	db	$34
	db	$0A
	db	$36
	db	$0A
	db	$37
	db	$0A
	db	$B0
	db	$35
	db	$3A
	db	$0A
	db	$37
	db	$0A
	db	$40
	db	$0A
	db	$37
	db	$0A
	db	$42
	db	$0A
	db	$37
	db	$0A
	db	$32
	db	$0A
	db	$B0
	db	$3D
	db	$42
	db	$0A
	db	$32
	db	$0A
	db	$30
	db	$0A
	db	$B0
	db	$41
	db	$40
	db	$0A
	db	$30
	db	$0A
	db	$2B
	db	$0A
	db	$B0
	db	$45
	db	$3B
	db	$0A
	db	$2B
	db	$0A
	db	$B0
	db	$48
	db	$6F
	db	$0A
	db	$2B
	db	$05
	db	$31
	db	$05
	db	$32
	db	$05
	db	$33
	db	$05
	db	$34
	db	$0A
	db	$B0
	db	$4F
	db	$37
	db	$0A
	db	$34
	db	$0A
	db	$3B
	db	$0A
	db	$2B
	db	$14
	db	$29
	db	$14
	db	$29
	db	$14
	db	$29
	db	$0A
	db	$29
	db	$14
	db	$29
	db	$14
	db	$29
	db	$05
	db	$28
	db	$05
	db	$29
	db	$05
	db	$31
	db	$05
	db	$34
	db	$05
	db	$B0
	db	$5C
	db	$39
	db	$05
	db	$31
	db	$05
	db	$34
	db	$05
	db	$39
	db	$05
	db	$41
	db	$05
	db	$B0
	db	$62
	db	$44
	db	$0A
	db	$39
	db	$0A
	db	$29
	db	$0A
	db	$39
	db	$0A
	db	$B1
	db	$68
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$B2
	db	$6B
	db	$27
	db	$0A
	db	$37
	db	$0A
	db	$B2
	db	$6E
	db	$22
	db	$0A
	db	$32
	db	$0A
	db	$B2
	db	$71
	db	$25
	db	$0A
	db	$35
	db	$0A
	db	$B2
	db	$74
	db	$29
	db	$0A
	db	$39
	db	$0A
	db	$B2
	db	$77
	db	$29
	db	$0A
	db	$6F
	db	$14
	db	$29
	db	$0A
	db	$2B
	db	$14
	db	$31
	db	$14
	db	$BF
	db	$00
unk_FB61:
	db	$64
	db	$07
	db	$FF
unk_FB64:
	db	$0B
	db	$30
	db	$06
	db	$48
	db	$09
	db	$18
	db	$10
	db	$18
	db	$0B
	db	$18
	db	$12
	db	$30
	db	$11
	db	$30
	db	$09
	db	$18
	db	$0A
	db	$48
	db	$BF
	db	$00
unk_FB78:
	db	$04
	db	$0C
	db	$04
	db	$0C
	db	$06
	db	$18
	db	$BF
	db	$00
unk_FB80:
	db	$52
	db	$03
	db	$4B
	db	$05
	db	$54
	db	$05
	db	$FF
unk_FB87:
	db	$20
	db	$05
	db	$25
	db	$05
	db	$40
	db	$02
	db	$45
	db	$02
	db	$50
	db	$02
	db	$45
	db	$02
	db	$40
	db	$02
	db	$45
	db	$02
	db	$40
	db	$02
	db	$30
	db	$02
	db	$35
	db	$02
	db	$30
	db	$02
	db	$25
	db	$02
	db	$15
	db	$05
	db	$10
	db	$05
	db	$FF
unk_FBA6:
	db	$36
	db	$03
	db	$37
	db	$03
	db	$36
	db	$03
	db	$37
	db	$03
	db	$39
	db	$03
	db	$39
	db	$03
	db	$FF
unk_FBB3:
	db	$30
	db	$05
	db	$FF
unk_FBB6:
	db	$60
	db	$05
	db	$FF
unk_FBB9:
	db	$0B
	db	$01
	db	$06
	db	$01
	db	$03
	db	$03
	db	$FF
unk_FBC0:
	db	$0B
	db	$01
	db	$02
	db	$01
	db	$0A
	db	$01
	db	$B4
	db	$00
	db	$FF
unk_FBC9:
	db	$6F
	db	$01
	db	$FF
unk_FBCC:
	db	$12
	db	$0C
	db	$12
	db	$0C
	db	$22
	db	$0C
	db	$B1
	db	$00
	db	$FF
unk_FBD5:
	db	$6F
	db	$60
	db	$6F
	db	$60
	db	$6F
	db	$18
	db	$34
	db	$24
	db	$30
	db	$0C
	db	$32
	db	$0C
	db	$29
	db	$54
	db	$6F
	db	$18
	db	$6F
	db	$18
	db	$30
	db	$18
	db	$2B
	db	$18
	db	$27
	db	$0C
	db	$29
	db	$54
	db	$6F
	db	$18
	db	$B0
	db	$02
	db	$6F
	db	$18
	db	$25
	db	$24
	db	$29
	db	$18
	db	$30
	db	$0C
	db	$2B
	db	$48
	db	$6F
	db	$18
	db	$6F
	db	$18
	db	$2A
	db	$24
	db	$32
	db	$18
	db	$35
	db	$0C
	db	$34
	db	$3C
	db	$34
	db	$0C
	db	$35
	db	$0C
	db	$37
	db	$0C
	db	$A0
	db	$5F
	db	$39
	db	$A8
	db	$6F
	db	$18
	db	$A0
	db	$4F
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$0C
	db	$24
	db	$0C
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$17
	db	$0C
	db	$19
	db	$0C
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$0C
	db	$24
	db	$0C
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$22
	db	$06
	db	$20
	db	$06
	db	$19
	db	$06
	db	$17
	db	$0C
	db	$19
	db	$0C
	db	$BF
	db	$00
unk_FC89:
	db	$29
	db	$0C
	db	$29
	db	$0C
	db	$37
	db	$06
	db	$39
	db	$06
	db	$29
	db	$0C
	db	$32
	db	$06
	db	$34
	db	$06
	db	$29
	db	$0C
	db	$37
	db	$0C
	db	$39
	db	$0C
	db	$B0
	db	$00
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$0C
	db	$29
	db	$0C
	db	$37
	db	$06
	db	$39
	db	$06
	db	$29
	db	$0C
	db	$32
	db	$06
	db	$34
	db	$06
	db	$29
	db	$0C
	db	$37
	db	$0C
	db	$39
	db	$0C
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$06
	db	$6F
	db	$06
	db	$27
	db	$06
	db	$6F
	db	$06
	db	$27
	db	$06
	db	$6F
	db	$06
	db	$27
	db	$06
	db	$6F
	db	$06
	db	$27
	db	$06
	db	$6F
	db	$06
	db	$29
	db	$0C
	db	$29
	db	$0C
	db	$37
	db	$06
	db	$39
	db	$06
	db	$29
	db	$0C
	db	$32
	db	$06
	db	$34
	db	$06
	db	$29
	db	$0C
	db	$37
	db	$0C
	db	$39
	db	$0C
	db	$B0
	db	$0B
	db	$6F
	db	$18
	db	$30
	db	$18
	db	$30
	db	$0C
	db	$35
	db	$18
	db	$39
	db	$0C
	db	$37
	db	$48
	db	$6F
	db	$30
	db	$35
	db	$24
	db	$3A
	db	$18
	db	$42
	db	$0C
	db	$40
	db	$3C
	db	$40
	db	$0C
	db	$40
	db	$0C
	db	$40
	db	$0C
	db	$41
	db	$A8
	db	$6F
	db	$18
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$0C
	db	$34
	db	$0C
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$27
	db	$0C
	db	$29
	db	$0C
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$0C
	db	$34
	db	$0C
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$32
	db	$06
	db	$30
	db	$06
	db	$29
	db	$06
	db	$27
	db	$0C
	db	$29
	db	$0C
	db	$BF
	db	$00
unk_FD9B:
	db	$09
	db	$0C
	db	$09
	db	$0C
	db	$17
	db	$06
	db	$19
	db	$06
	db	$09
	db	$0C
	db	$12
	db	$06
	db	$14
	db	$06
	db	$09
	db	$0C
	db	$17
	db	$0C
	db	$19
	db	$0C
	db	$B0
	db	$00
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$05
	db	$06
	db	$6F
	db	$06
	db	$05
	db	$06
	db	$05
	db	$06
	db	$05
	db	$06
	db	$6F
	db	$06
	db	$05
	db	$06
	db	$05
	db	$06
	db	$07
	db	$06
	db	$6F
	db	$06
	db	$07
	db	$06
	db	$07
	db	$06
	db	$07
	db	$06
	db	$6F
	db	$06
	db	$07
	db	$06
	db	$07
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$B0
	db	$0B
	db	$05
	db	$06
	db	$6F
	db	$06
	db	$05
	db	$06
	db	$05
	db	$06
	db	$B2
	db	$4C
	db	$07
	db	$06
	db	$6F
	db	$06
	db	$07
	db	$06
	db	$07
	db	$06
	db	$B2
	db	$51
	db	$0A
	db	$06
	db	$6F
	db	$06
	db	$0A
	db	$06
	db	$0A
	db	$06
	db	$B2
	db	$56
	db	$10
	db	$06
	db	$6F
	db	$06
	db	$10
	db	$06
	db	$10
	db	$06
	db	$B2
	db	$5B
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$B0
	db	$60
	db	$09
	db	$06
	db	$6F
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$B0
	db	$69
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$09
	db	$06
	db	$B0
	db	$6E
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$0C
	db	$14
	db	$0C
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$12
	db	$06
	db	$10
	db	$06
	db	$09
	db	$06
	db	$07
	db	$0C
	db	$09
	db	$0C
	db	$B0
	db	$73
	db	$BF
	db	$00
unk_FEBD:
	db	$06
	db	$0C
	db	$06
	db	$0C
	db	$6F
	db	$18
	db	$6F
	db	$30
	db	$B0
	db	$00
	db	$03
	db	$0C
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$0C
	db	$06
	db	$0C
	db	$03
	db	$0C
	db	$B6
	db	$05
	db	$03
	db	$06
	db	$6F
	db	$06
	db	$03
	db	$06
	db	$03
	db	$06
	db	$BE
	db	$0F
	db	$03
	db	$0C
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$0C
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$0C
	db	$03
	db	$0C
	db	$06
	db	$06
	db	$B6
	db	$23
	db	$06
	db	$06
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$06
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$06
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$06
	db	$03
	db	$06
	db	$03
	db	$06
	db	$06
	db	$0C
	db	$06
	db	$0C
	db	$B2
	db	$25
	db	$BF
	db	$00
unk_FF27:
	db	$57
	db	$08
	db	$FF
unk_FF2A:
	db	$25
	db	$0A
	db	$20
	db	$08
	db	$15
	db	$04
	db	$10
	db	$02
	db	$FF
unk_FF33:
	db	$50
	db	$05
	db	$60
	db	$04
	db	$57
	db	$03
	db	$56
	db	$02
	db	$54
	db	$01
	db	$53
	db	$01
	db	$52
	db	$01
	db	$51
	db	$01
	db	$50
	db	$01
	db	$FF
unk_FF46:
	db	$20
	db	$02
	db	$20
	db	$01
	db	$42
	db	$07
	db	$FF
unk_FF4D:
	db	$0B
	db	$02
	db	$05
	db	$02
	db	$0A
	db	$02
	db	$04
	db	$02
	db	$09
	db	$02
	db	$03
	db	$02
	db	$FF
unk_FF5A:
	db	$45
	db	$04
	db	$A0
	db	$97
	db	$42
	db	$04
	db	$A0
	db	$98
	db	$40
	db	$04
	db	$A0
	db	$99
	db	$3B
	db	$04
	db	$A0
	db	$9B
	db	$37
	db	$04
	db	$A0
	db	$9C
	db	$35
	db	$04
	db	$34
	db	$04
	db	$A0
	db	$9F
	db	$30
	db	$04
	db	$FF
unk_FF77:
	db	$22
	db	$01
	db	$20
	db	$01
	db	$19
	db	$01
	db	$17
	db	$01
	db	$15
	db	$01
	db	$14
	db	$01
	db	$12
	db	$01
	db	$FF
unk_FF86:
	db	$32
	db	$04
	db	$30
	db	$04
	db	$B2
	db	$00
	db	$FF
	db	$32
	db	$04
	db	$30
	db	$04
	db	$B2
	db	$00
	db	$FF
	db	$E4
	db	$00
	db	$00
	db	$BF
	db	$FF
	db	$00
	db	$1C
	db	$00
	db	$00
	db	$00
	db	$00
	db	$00
	db	$1C
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00
	db	$FF
	db	$00